@@ -2,7 +2,7 @@
  * FileTranscoder interface for MP3FS
  *
  * Copyright (C) 2006-2008 David Collett
- * Copyright (C) 2008-2010 Kristofer Henriksson
+ * Copyright (C) 2008-2011 Kristofer Henriksson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@ struct mp3fs_params {
     unsigned int bitrate;
     unsigned int quality;
     int debug;
+    int gainmode;
+    float gainref;
 };
 
 #define mp3fs_debug(f, ...) syslog(LOG_DEBUG, f, ## __VA_ARGS__)
